@@ -1,4 +1,4 @@
-# Copyright 2019-2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright Amazon.com Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -13,7 +13,6 @@
 
 from __future__ import annotations
 
-import itertools
 from abc import ABC, abstractmethod
 from functools import singledispatch
 from typing import Any, Dict, List, Optional, Union
@@ -21,7 +20,7 @@ from typing import Any, Dict, List, Optional, Union
 import numpy as np
 from braket.ir import jaqcd
 
-from braket.default_simulator.density_matrix_simulation import DensityMatrixSimulation
+from braket.default_simulator.linalg_utils import marginal_probability, partial_trace
 from braket.default_simulator.observables import (
     Hadamard,
     Hermitian,
@@ -33,6 +32,7 @@ from braket.default_simulator.observables import (
 )
 from braket.default_simulator.operation import Observable
 from braket.default_simulator.operation_helpers import ir_matrix_to_ndarray
+from braket.default_simulator.simulation import Simulation
 from braket.default_simulator.state_vector_simulation import StateVectorSimulation
 
 
@@ -46,7 +46,7 @@ def from_braket_result_type(result_type) -> ResultType:
         ResultType: Instance of specific `ResultType` corresponding to the type of result_type
 
     Raises:
-        ValueError: If no concrete `ResultType` class has been registered
+        NotImplementedError: If no concrete `ResultType` class has been registered
             for the Braket instruction type
     """
     return _from_braket_result_type(result_type)
@@ -54,26 +54,30 @@ def from_braket_result_type(result_type) -> ResultType:
 
 @singledispatch
 def _from_braket_result_type(result_type):
-    raise ValueError(f"Result type {result_type} not recognized")
+    raise NotImplementedError(f"Result type {result_type} not recognized")
 
 
 class ResultType(ABC):
     """
     An abstract class that when implemented defines a calculation on a
     quantum state simulation.
+
+    Note:
+        All result types are calculated exactly, instead of approximated from samples.
+        Sampled results are returned from `Simulation.retrieve_samples`, which can be processed by,
+        for example, the Amazon Braket SDK.
     """
 
     @abstractmethod
-    def calculate(self, simulation: StateVectorSimulation) -> Any:
-        # Return type of any due to lack of sum type support in Python
+    def calculate(self, simulation: Simulation) -> Any:
+        # Return type of Any due to lack of sum type support in Python
         """Calculate a result from the given quantum state vector simulation.
 
         Args:
-            simulation (StateVectorSimulation): The quantum state vector simulation
-                to use in the calculation
+            simulation (Simulation): The simulation to use in the calculation.
 
         Returns:
-            Any: The result of the calculation
+            Any: The result of the calculation.
         """
 
 
@@ -94,57 +98,40 @@ class ObservableResultType(ResultType, ABC):
         """Observable: The observable for which the desired result is calculated."""
         return self._observable
 
-    def calculate(self, simulation: StateVectorSimulation) -> Union[float, List[float]]:
-        probabilities = simulation._probabilities(simulation.state_with_observables)
-        qubit_count = simulation.qubit_count
-        eigenvalues = self._observable.eigenvalues
-        targets = self._observable.measured_qubits
-        if targets:
-            return ObservableResultType._calculate_for_targets(
-                probabilities,
-                qubit_count,
-                targets,
-                eigenvalues,
-                self._calculate_from_prob_distribution,
-            )
-        else:
-            return [
-                ObservableResultType._calculate_for_targets(
-                    probabilities,
-                    qubit_count,
-                    [i],
-                    eigenvalues,
-                    self._calculate_from_prob_distribution,
-                )
-                for i in range(qubit_count)
-            ]
+    def calculate(self, simulation: Simulation) -> Union[float, List[float]]:
+        """Calculates the result type using the underlying observable.
+
+        Returns a real number if the observable has defined targets,
+        or a list of real numbers, one for the result type on each target,
+        if the observable has no target.
+
+        Args:
+            simulation (Simulation): The simulation to use in the calculation.
+
+        Returns:
+            Union[float, List[float]]: The value of the result type;
+            will be a real due to self-adjointness of observable.
+        """
+        if self._observable.measured_qubits:
+            return self._calculate_single_quantity(simulation, self._observable)
+        return [
+            self._calculate_single_quantity(simulation, self._observable.fix_qubit(qubit))
+            for qubit in range(simulation.qubit_count)
+        ]
 
     @staticmethod
     @abstractmethod
-    def _calculate_from_prob_distribution(
-        probabilities: np.ndarray, eigenvalues: np.ndarray
-    ) -> float:
-        """Calculates a result from the probabilities of eigenvalues.
+    def _calculate_single_quantity(simulation: Simulation, observable: Observable) -> float:
+        """Calculates a single real value of the result type.
 
         Args:
-            probabilities (np.ndarray): The probability of measuring each eigenstate
-            eigenvalues (np.ndarray): The eigenvalue corresponding to each eigenstate
+            simulation (Simulation): The simulation to use in the calculation.
+            observable (Observable): The observable used to calculate the result type.
 
         Returns:
-            float: The result of the calculation
+            float: The value of the result type.
         """
 
-    @staticmethod
-    def _calculate_for_targets(
-        probabilities,
-        qubit_count,
-        targets,
-        eigenvalues,
-        calculate_from_prob_distribution,
-    ):
-        prob = _marginal_probability(probabilities, qubit_count, targets)
-        return calculate_from_prob_distribution(prob, eigenvalues)
-
 
 class StateVector(ResultType):
     """
@@ -164,7 +151,7 @@ class StateVector(ResultType):
 
 
 @_from_braket_result_type.register
-def _(statevector: jaqcd.StateVector):
+def _(_: jaqcd.StateVector):
     return StateVector()
 
 
@@ -182,30 +169,32 @@ class DensityMatrix(ResultType):
         """
         self._targets = targets
 
-    def calculate(self, simulation: DensityMatrixSimulation) -> np.ndarray:
+    def calculate(self, simulation: Simulation) -> np.ndarray:
         """Return the given density matrix of the simulation.
 
         Args:
-            simulation (DensityMatrixSimulation): The simulation whose (full or reduced)
+            simulation (Simulation): The simulation whose (full or reduced)
                 density matrix will be returned.
 
         Returns:
             np.ndarray: The density matrix (before observables) of the simulation
         """
-        if self._targets is None or self._targets == list(range(simulation.qubit_count)):
+        if self._targets is None or len(self._targets) == simulation.qubit_count:
             return simulation.density_matrix
         else:
             if not all(ta in list(range(simulation.qubit_count)) for ta in self._targets):
                 raise IndexError(
                     "Input target qubits must be within the range of the qubits in the circuit."
                 )
-
-            return _partial_trace(simulation.density_matrix, simulation.qubit_count, self._targets)
+            return partial_trace(
+                simulation.density_matrix.reshape(np.array([2] * 2 * simulation.qubit_count)),
+                self._targets,
+            )
 
 
 @_from_braket_result_type.register
-def _(densitymatrix: jaqcd.DensityMatrix):
-    return DensityMatrix(densitymatrix.targets)
+def _(density_matrix: jaqcd.DensityMatrix):
+    return DensityMatrix(density_matrix.targets)
 
 
 class Amplitude(ResultType):
@@ -255,23 +244,21 @@ class Probability(ResultType):
         """
         self._targets = targets
 
-    def calculate(self, simulation: StateVectorSimulation) -> np.ndarray:
+    def calculate(self, simulation: Simulation) -> np.ndarray:
         """Return the marginal probabilities of computational basis states on the target qubits.
 
         Probabilities are marginalized over all non-target qubits.
 
         Args:
-            simulation (StateVectorSimulation): The simulation from which probabilities
-                are calculated
+            simulation (Simulation): The simulation from which probabilities are calculated.
 
         Returns:
             np.ndarray: An array of probabilities of length equal to 2^(number of target qubits),
             indexed by the decimal encoding of the computational basis state on the target qubits
 
         """
-        return _marginal_probability(
+        return marginal_probability(
             simulation.probabilities,
-            simulation.qubit_count,
             self._targets,
         )
 
@@ -286,18 +273,9 @@ class Expectation(ObservableResultType):
     Holds an observable :math:`O` to calculate its expected value.
     """
 
-    def __init__(self, observable: Observable):
-        """
-        Args:
-            observable (Observable): The observable for which expected value is calculated
-        """
-        super().__init__(observable)
-
     @staticmethod
-    def _calculate_from_prob_distribution(
-        probabilities: np.ndarray, eigenvalues: np.ndarray
-    ) -> float:
-        return (probabilities @ eigenvalues).real
+    def _calculate_single_quantity(simulation: Simulation, observable: Observable) -> float:
+        return simulation.expectation(observable)
 
 
 @_from_braket_result_type.register
@@ -310,18 +288,9 @@ class Variance(ObservableResultType):
     Holds an observable :math:`O` to calculate its variance.
     """
 
-    def __init__(self, observable: Observable):
-        """
-        Args:
-            observable (Observable): The observable for which variance is calculated
-        """
-        super().__init__(observable)
-
     @staticmethod
-    def _calculate_from_prob_distribution(
-        probabilities: np.ndarray, eigenvalues: np.ndarray
-    ) -> float:
-        return probabilities @ (eigenvalues.real ** 2) - (probabilities @ eigenvalues).real ** 2
+    def _calculate_single_quantity(simulation: Simulation, observable: Observable) -> float:
+        return simulation.expectation(observable ** 2) - simulation.expectation(observable) ** 2
 
 
 @_from_braket_result_type.register
@@ -381,53 +350,3 @@ def _actual_targets(targets: List[int], num_qubits: int, is_factor: bool):
         return [targets.pop(0) for _ in range(num_qubits)]
     except Exception:
         raise ValueError("Insufficient qubits for tensor product")
-
-
-def _marginal_probability(
-    probabilities: np.ndarray,
-    qubit_count: int,
-    targets: List[int] = None,
-) -> np.ndarray:
-    """Return the marginal probability of the computational basis states.
-
-    The marginal probability is obtained by summing the probabilities on
-    the unused qubits. If no targets are specified, then the probability
-    of all basis states is returned.
-    """
-
-    if targets is None or targets == list(range(qubit_count)):
-        # All qubits targeted, no need to marginalize
-        return probabilities
-
-    targets = np.hstack(targets)
-
-    # Find unused qubits and sum over them
-    unused_qubits = list(set(range(qubit_count)) - set(targets))
-    as_tensor = probabilities.reshape([2] * qubit_count)
-    marginal = np.apply_over_axes(np.sum, as_tensor, unused_qubits).flatten()
-
-    # Reorder qubits to match targets
-    basis_states = np.array(list(itertools.product([0, 1], repeat=len(targets))))
-    perm = np.ravel_multi_index(
-        basis_states[:, np.argsort(np.argsort(targets))].T, [2] * len(targets)
-    )
-    return marginal[perm]
-
-
-def _partial_trace(
-    density_matrix: np.ndarray,
-    qubit_count: int,
-    targets: List[int],
-) -> np.ndarray:
-    """Return the reduced density matrix for the target qubits."""
-
-    targets = np.asarray(targets)
-    dims = np.asarray([2] * qubit_count)
-    Ndim = dims.size
-    Nkeep = np.prod(dims[targets])
-
-    idx1 = [i for i in range(Ndim)]
-    idx2 = [Ndim + i if i in targets else i for i in list(range(Ndim))]
-    tr_rho = density_matrix.reshape(np.tile(dims, 2))
-    tr_rho = np.einsum(tr_rho, idx1 + idx2)
-    return tr_rho.reshape(Nkeep, Nkeep)
