@@ -1,4 +1,4 @@
-# Copyright 2019-2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright Amazon.com Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -70,7 +70,6 @@ class BaseLocalSimulator(BraketSimulator):
                 instructions to execute.
             qubit_count (int): The number of qubits to simulate.
             shots (int): The number of times to run the circuit.
-            simulation (Simulation): Simulation method for evolving the state.
             batch_size (int): The size of the circuit partitions to contract,
                 if applying multiple gates at a time is desired; see `StateVectorSimulation`.
                 Must be a positive integer.
@@ -81,7 +80,7 @@ class BaseLocalSimulator(BraketSimulator):
 
         Raises:
             ValueError: If result types are not specified in the IR or sample is specified
-                as a result type when shots=0. Or, if statevector and amplitude result types
+                as a result type when shots=0. Or, if StateVector and Amplitude result types
                 are requested when shots>0.
         """
         self._validate_ir_results_compatibility(circuit_ir)
@@ -110,14 +109,13 @@ class BaseLocalSimulator(BraketSimulator):
                 non_observable_result_types,
                 observable_result_types,
             ) = BaseLocalSimulator._translate_result_types(circuit_ir)
-            observables = BaseLocalSimulator._validate_and_consolidate_observable_result_types(
+            BaseLocalSimulator._validate_and_consolidate_observable_result_types(
                 list(observable_result_types.values()), qubit_count
             )
             results = BaseLocalSimulator._generate_results(
                 circuit_ir,
                 non_observable_result_types,
                 observable_result_types,
-                observables,
                 simulation,
             )
 
@@ -151,7 +149,7 @@ class BaseLocalSimulator(BraketSimulator):
                 if name not in supported_instructions:
                     raise TypeError(
                         'Noise instructions are not supported by the state vector simulator (by default). \
-You need to use the density matrix simualtor: LocalSimulator("braket_dm").'
+You need to use the density matrix simulator: LocalSimulator("braket_dm").'
                     )
         if no_noise and _NOISE_INSTRUCTIONS.intersection(supported_instructions):
             warnings.warn(
@@ -383,7 +381,6 @@ for a better user experience.'
         circuit_ir: Program,
         non_observable_result_types: Dict[int, ResultType],
         observable_result_types: Dict[int, ObservableResultType],
-        observables: List[Observable],
         simulation,
     ) -> List[ResultTypeValue]:
 
@@ -396,7 +393,6 @@ for a better user experience.'
             )
 
         if observable_result_types:
-            simulation.apply_observables(observables)
             for index in observable_result_types:
                 results[index] = ResultTypeValue.construct(
                     type=circuit_ir.results[index],
@@ -442,13 +438,15 @@ for a better user experience.'
 
         return GateModelTaskResult.construct(**result_dict)
 
-    @property
-    def simulation_type(self):
-        raise NotImplementedError("simulation_type has not been implemented yet.")
-
     @property
     def properties(self) -> GateModelSimulatorDeviceCapabilities:
-        """properties of simulator such as supported IR types, quantum operations,
-        and result types.
+        """GateModelSimulatorDeviceCapabilities: Properties of simulator such as supported IR types,
+        quantum operations, and result types.
         """
-        raise NotImplementedError("properties has not been implemented yet.")
+        raise NotImplementedError("properties has not been implemented.")
+
+    def initialize_simulation(self, **kwargs) -> Simulation:
+        """
+        Initializes simulation with keyword arguments
+        """
+        raise NotImplementedError("initialize_simulation has not been implemented.")
